function EQ=autoEQ(connection, ex_trq, CL_connection, CL_state, connection_pre, vars,states)
%edited 201806271342
%edited 201707041129
%edited 201703071225
%edited 201611201402
%edited 201611171148
%edited 201611151129
%edited 201611141318
%edited 201611131852
%edited 201611131714


%input tests

if (nargin<4)
    error('not enough input')
else if (nargin==4)
        connection_C=ones(size(connection));
        known_vars=[];
        unknown_vars=[];
    else
        if (nargin>7)
            error('too much input')
        else
            if ~isempty(connection_pre)
                connection_C=connection_pre;
            else
                connection_C=ones(size(connection));
            end
            if (nargin >= 6 && ~isempty(vars))
                known_vars=vars(1,:);
                unknown_vars=vars(2,:);
            else
                known_vars=[];
                unknown_vars=[];
            end
        end
    end
end

if (size(connection,1)~=3)
    error('connection matrix size error')
end

if (sum(connection(:,1))~=0)
    error('elements of the first column of connection matrix should all be 0')
end

if (size(ex_trq,1)~=3)
    error('external torque matrix size error')
end

if (size(CL_connection,1)~=4 && ~isempty(CL_connection))
    error('clutch connection matrix size error')
end

if (size(CL_connection,2)~=length(CL_state))
    error('number of columns of clutch connection matrix should be equal to length of clutch state vector')
end

if (size(connection,2)~=size(ex_trq,2))
    error('number of columns of connection matrix and external torque matrix should be equal')
end
%end of input tests

%number of planetary gear sets
num_PG=size(connection,2);

%number of clutches
num_CL=size(CL_connection,2);

%number of external torques
ind_exT=find(ex_trq);
num_exT=length(ind_exT);

%number of internal torques between PGs
ind_inT=find(connection);
num_inT=length(ind_inT);

num_vars=size(known_vars,2);

str_a={'aR','aC','aS'};
str_c={'R','C','S'};
num_c=cell(1,num_PG);
for i=1:num_PG
    num_c{i}=num2str(i);
end
str_J={'JR','JC','JS'};

%declare symbol variables
a_matrix=sym(zeros(3,num_PG));
J_matrix=sym(zeros(3,num_PG));
k_vector=sym(zeros(1,num_PG));
Ti_vector=sym(zeros(1,num_PG));
for i=1:num_PG
    for j=1:3
        a_matrix(j,i)=sym(strcat(str_a(j),num2str(i)),'real');
        J_matrix(j,i)=sym(strcat(str_J(j),num2str(i)),'real');
    end
    k_vector(i)=sym(strcat(('k'),num2str(i)),'real');
    Ti_vector(i)=sym(strcat(('Ti'),num2str(i)),'real');
end


TCL_vector=sym(zeros(1,num_CL));
TCLx_vector=sym(zeros(1,num_CL));
%k for known
TCLk_vector=sym(zeros(1,num_CL));
%u for unknown
TCLu_vector=sym(zeros(1,num_CL));
TCL_matrix=sym(zeros(3,num_PG));
CL_Tin_vector=sym(zeros(1,num_CL));
CL_Tout_vector=sym(zeros(1,num_CL));
for i=1:num_CL
    TCL_vector(i)=sym(strcat('TCL',num2str(i)),'real');
    TCLx_vector(i)=sym(strcat('TCL_',num2str(i)),'real');
    if (CL_state(i)==1)
        TCLk_vector(i)=TCLx_vector(i);
        TCLu_vector(i)=TCL_vector(i);
    else
        TCLk_vector(i)=TCL_vector(i);
        TCLu_vector(i)=TCLx_vector(i);
    end
    CL_Tin_vector(i)=sym(strcat('CL',num2str(i),'_Tin'),'real');
    CL_Tout_vector(i)=sym(strcat('CL',num2str(i),'_Tout'),'real');
    node1_c=CL_connection(1,i);
    node1_r=CL_connection(2,i);
    node2_c=CL_connection(3,i);
    node2_r=CL_connection(4,i);
    if (node1_r~=0 && node1_c~=0) %not braker
        TCL_matrix(node1_r,node1_c)=TCL_matrix(node1_r,node1_c)-TCL_vector(i);
    end
    TCL_matrix(node2_r,node2_c)=TCL_matrix(node2_r,node2_c)+TCL_vector(i);
    
    
end



[I, V]=ind2sub(size(connection),ind_inT);
inT_vector=sym(zeros(1,num_inT));
inT_matrix=sym(zeros(3,num_PG));
for i = 1:num_inT
    %row index: I(i)
    %column index: V(i)
    %row index of previous: connection(ind_inT(i))
    %column index of previous: V(i)-connection_C(I(i),V(i))
    pre_c=strcat(str_c{connection(ind_inT(i))},num_c{V(i)-connection_C(I(i),V(i))});
    next_c=strcat(str_c{I(i)},num_c{V(i)});
    str_inT=strcat(pre_c,next_c);
    inT_vector(i)=sym(strcat('T_',str_inT),'real');
    inT_matrix(connection(ind_inT(i)),V(i)-connection_C(I(i),V(i)))=inT_matrix(connection(ind_inT(i)),V(i)-connection_C(I(i),V(i)))-inT_vector(i);
    inT_matrix(I(i),V(i))=inT_matrix(I(i),V(i))+inT_vector(i);
end

[I_exT,V_exT]=ind2sub(size(ex_trq),ind_exT);
exT_vector=sym(zeros(1,num_exT));
exT_matrix=sym(zeros(3,num_PG));
for i=1:num_exT
    exT_vector(i)=sym(strcat('Tex',num2str(i)),'real');
    exT_matrix(I_exT(i),V_exT(i))=exT_vector(i);
end


xCL_matrix=sym(zeros(3,num_PG));
% xCL results from 3 parts:
% 1.exT; 2.inT; 3.Ti;
for i=1:num_PG
    for j=1:3
        switch j
            case 1
                xCL_matrix(j,i)=exT_matrix(j,i)+inT_matrix(j,i)+Ti_vector(i)*k_vector(i)/(1+k_vector(i));
                
            case 2
                xCL_matrix(j,i)=exT_matrix(j,i)+inT_matrix(j,i)-Ti_vector(i);
                
            case 3
                xCL_matrix(j,i)=exT_matrix(j,i)+inT_matrix(j,i)+Ti_vector(i)/(1+k_vector(i));
                
        end
    end
end

T1_vector=sym(zeros(num_CL,1));
T2_vector=sym(zeros(num_CL,1));
for i=1:num_CL
    node1_c=CL_connection(1,i);
    node1_r=CL_connection(2,i);
    node2_c=CL_connection(3,i);
    node2_r=CL_connection(4,i);
    
    T2_vector(i)=xCL_matrix(node2_r,node2_c)+TCL_matrix(node2_r,node2_c)-TCL_vector(i);
    
    if (node1_r~=0 && node1_c~=0) %not braker
        T1_vector(i)=xCL_matrix(node1_r,node1_c)+TCL_matrix(node1_r,node1_c)+TCL_vector(i);
    else
        T1_vector(i)=-T2_vector(i);
    end
end

%end of declaration of symbol variables

ind_lockedCL=find(CL_state);
num_lockedCL=length(ind_lockedCL);
%num_eqs=4*num_PG+num_lockedCL+num_inT;
EQs=[];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%formulate equations
for i=1:num_PG
    EQs=[EQs;SinglePG(J_matrix(1,i),J_matrix(2,i),J_matrix(3,i),k_vector(i),[a_matrix(:,i);Ti_vector(i)],[exT_matrix(:,i)+inT_matrix(:,i)+TCL_matrix(:,i);0])];
end

% for locked clutches, there are two ways of formulating equations
% option #1 using the velocity relationship
% option #2 using the torque relationship

% option #2
% for i=1:num_lockedCL
%     id=ind_lockedCL(i);
%     node1_c=CL_connection(1,id);
%     node1_r=CL_connection(2,id);
%     node2_c=CL_connection(3,id);
%     node2_r=CL_connection(4,id);
%
%     T1=T1_vector(id);
%     T2=T2_vector(id);
%
%     if (node1_r~=0 && node1_c~=0) %not braker
%         J1=J_matrix(node1_r,node1_c);
%     end
%
%     J2=J_matrix(node2_r,node2_c);
%
%     if (node1_r~=0 && node1_c~=0) %not braker
%         TCL_express=(T1*J2-T2*J1)/(J1+J2);
%     else
%         TCL_express=-T2;
%     end
%
%     EQs=[EQs;TCL_vector(id)-TCL_express];
%
% end

% option #1
for i=1:num_lockedCL
    id=ind_lockedCL(i);
    node1_c=CL_connection(1,id);
    node1_r=CL_connection(2,id);
    node2_c=CL_connection(3,id);
    node2_r=CL_connection(4,id);
    
    if (node1_r~=0 && node1_c~=0) %not braker
        EQs=[EQs;a_matrix(node1_r,node1_c)-a_matrix(node2_r,node2_c)];
    else
        EQs=[EQs;a_matrix(node2_r,node2_c)];
    end
    
end

for i=1:num_inT
    %row index: I(i)
    %column index: V(i)
    %row index of previous: connection(ind_inT(i))
    %column index of previous: V(i)-connection_C(I(i),V(i))
    a1=a_matrix(connection(ind_inT(i)), V(i)-connection_C(I(i),V(i)));
    a2=a_matrix(I(i), V(i));
    EQs=[EQs;a1-a2];
end



EQs1=EQs;
for i=1:num_CL
    EQs1=[EQs1;CL_Tin_vector(i)-T1_vector(i)];
    EQs1=[EQs1;CL_Tout_vector(i)-T2_vector(i)];
    if (CL_state(i)==0)
        EQs1=[EQs1;TCLx_vector(i)-TCL_vector(i)];
    end
end


a_vector=reshape(a_matrix,1,numel(a_matrix));






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% theory
unknowns=[a_vector,Ti_vector,TCL_vector(ind_lockedCL),inT_vector];


knowns=[exT_vector,TCL_vector(CL_state==0)];

DOF=2*num_PG-num_inT-num_lockedCL;
disp(['DOF is',' ', num2str(DOF)])

if nargin==7
    
    [Lia, Locb]=ismember(states, a_vector);
    if sum(Lia)<length(Lia)
        error('State variables should all be accelerations')
    end
    
    
    a_States=a_vector(Locb);
    
    a_notStates=a_vector(~ismember(a_vector,states));
    
    % Equations about rotational speed
    EQs_w = [EQs((1:num_PG)*4);EQs(num_PG*4+1:end)];
    [EQ.Aw,EQ.Cw]=equationsToMatrix(EQs_w,a_notStates);
    EQ.Bw=equationsToMatrix(EQ.Cw,a_States);
    
    EQ.wid=a_States;
    EQ.wd=a_notStates;
    
    % test state numbers
    if (length(states)~=DOF)
        error(['The number of state variables should be ',num2str(DOF)]);
    end
    
    % test dependency
    if rank(EQ.Aw)~=length(a_notStates)
        error('State variables should be independent')
    end
    
    a_vector_rearranged=[a_States, a_notStates];
    unknowns_rearranged=[a_vector_rearranged,Ti_vector,TCL_vector(ind_lockedCL),inT_vector];
    [~,temp2]=ismember(a_States, a_vector);
    temp4=ceil(temp2/3)-1+temp2;
    EQs_rearranged=[EQs(temp4);setdiff(EQs,EQs(temp4),'stable')];
    [EQ.A_rearranged,EQ.C_rearranged]=equationsToMatrix(EQs_rearranged, unknowns_rearranged);
    EQ.B_rearranged=equationsToMatrix(EQ.C_rearranged,knowns);
    temp=EQ.A_rearranged\EQ.B_rearranged;
    num_states=length(states);
    C2=temp((num_states+1):end,:);
    A11=EQ.A_rearranged(1:num_states,1:num_states);
    A12=EQ.A_rearranged(1:num_states,(num_states+1):end);
    B1=EQ.B_rearranged(1:num_states,:);
    EQ.As=A11;
    EQ.Bs=B1-A12*C2;
    EQ.Xs=a_States;
    EQ.Ys=knowns;
    EQ.Cs=temp(1:num_states,:);
    
end

if (numel(known_vars)~=0)
    ind_u=zeros(size(known_vars));
    ind_k=zeros(size(unknown_vars));
    for i = 1:num_vars
        ind_ui=find(unknowns==known_vars{i},1);
        if (isempty(ind_ui))
            error('%s is not within the unknown variables.\nCheck the specified clutch states.',known_vars{i});
        end
        ind_u(i)=ind_ui;
        
        ind_ki=find(knowns==unknown_vars{i},1);
        if (isempty(ind_ki))
            error('%s is not within the known variables.\nCheck the specified clutch states.',unknown_vars{i});
        end
        ind_k(i)=ind_ki;
    end
    temp=unknowns(ind_u);
    unknowns(ind_u)=knowns(ind_k);
    knowns(ind_k)=temp;
end

[EQ.A,EQ.C]=equationsToMatrix(EQs,unknowns);

EQ.B=equationsToMatrix(EQ.C,knowns);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% convenient for modeling
unknowns1=[a_vector,Ti_vector,TCLu_vector,inT_vector,CL_Tin_vector,CL_Tout_vector];

knowns1=[exT_vector,TCLk_vector];

if (numel(known_vars)~=0)
    ind_u=zeros(size(known_vars));
    ind_k=zeros(size(unknown_vars));
    for i = 1:num_vars
        ind_u(i)=find(unknowns1==known_vars{i});
        ind_k(i)=find(knowns1==unknown_vars{i});
    end
    temp=unknowns1(ind_u);
    unknowns1(ind_u)=knowns1(ind_k);
    knowns1(ind_k)=temp;
end

[EQ.A1,EQ.C1]=equationsToMatrix(EQs1,unknowns1);

EQ.B1=equationsToMatrix(EQ.C1,knowns1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





EQ.Y=knowns;
EQ.Y1=knowns1;
EQ.X=unknowns;
EQ.X1=unknowns1;

EQ.J=J_matrix;
EQ.k=k_vector;

EQ.eqs=EQs;
if (nargin == 7)
    EQ.eqs_rearranged=EQs_rearranged;
end
EQ.eqs1=EQs1;

EQ.TCL_in=T1_vector;
EQ.TCL_out=T2_vector;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%








